;=========================================================
; KGIF: a tiny GIF viewer. Beta.
;
; View a single GIF or all GIFs in the current directory.
; VESA.
;
; (c) 1996, 1997 - by Benny <benny_land@geocities.com>
;
; A86 version.
;
; NASM version:
; to get it to work with NASM: delete all 'ptr' and 'offset';
; lowercase(or uppercase) all variables as NASM is case sensitive;
; uncomment the times's and comment out the dup's.
;
; TASM version:
; uncomment the headers and footers;
; comment out the 'cs mov ax,[value]', uncomment the 'mov ax,cs:[value]'
; change the bank switching
;  from     'call    far [cs: window_bank_switch]'
;  to       'call    dword ptr[cs: window_bank_switch]'
;  *if* you use direct bank switching.
; Compile and link it to a .com file.
;
; Free for non-commercial use.
; Disclaimer: no warranties.
;=========================================================
major_version equ     '2'
minor_version equ     '0'
;internal_version:    classic
;---------------------------------------------------------
; - Originally called VGR.
;
; Changes for version 1.7:
; ========================
; - Better VESA detection
; - Detection of various granularities
; - Setcolor sets the background.
; - Help screen cyan/blue at (2,1)
; - Bank switching for winA & winB, test if exist+writable
; - ToggleCenter improved
; - VESA Debug info at the end when shift-Escape pressed
;   or during viewing with the 'v' key
; - Direct banking!
; Changes for version 1.8:
; ========================
; - TASM version
; - NASM version
; Changes for version 1.9:
; ========================
; - fixes in exit handling
; - check for GIF87a/89a signature
; - back to normal banking
; Changes for version 2.0:
; ========================
; - fix for getting into a loop if a Graphics Control Extension Block
; contained zero as data (fix by olavi.ostman@tietotehdas.fi)
; [Untested by me (Benny), I just put it in.]
;
; Bugs:
; =====
; - Direct bank switching doesn't work on some cards.
; - Doesn't handle the 32k/32k double window cards.
;
; - Choke on interlaced pictures (should be easy to implement:
; one needs to put back the video pointer on top of the screen
; plus 1-2-3-4 row(s) after each interlace run.)
; - Can't manage multiple pics per GIF
; - No panning
; - No check for nonsense width or height
;
; - The Help screen and the text is invisible in SVGA modes
; on a S3 Virge. And putpixel/getpixel bios calls are broken
; on the Virge (that makes the palette viewer unusable.)
; Temporary solution: zoom in until 320x200, then all text
; will be visible and the palette will be displayed.
;
; - The trash_palette function uses the bios timer. If the timer
; is too high/low a number, the palette will be all black.
; Temporary solution: wait a few seconds and try again.
;
;---------------------------------------------------------
; TASM stuff:
.286
;kgif            segment byte public
;                assume  cs:kgif, ds:kgif
                org     100h
start:
                jmp     Begin_The_Show
;------------
YES           EQU       1
NO            EQU       0
;------------
MAXIMUMFILES  EQU     200        ; Maximum files to read in a directory
CenterPIC     EQU     YES        ; center picture at start by default?
ToggleCenter  db      CenterPIC  ; program choice
ToggleCenter2 db      CenterPIC  ; user choice
ToggleTurn    db      NO         ; display off while loading by default?
ToggleCount   db      NO         ; count colors by default?
;------------

FILENAME    EQU         09EH    ; address of filename (findfirst/findnext)

; "times" is NASM sort of substitute for dup
;paramlist   times      256 db 0
;decoder_t2  times      256 db 0
;colorcount  times      256 db 1
;filestore   times      (13*MAXIMUMFILES) db 0
paramlist   db      256 dup (0)
decoder_t2  db      256 dup (0)
colorcount  db      256 dup (1)
filestore   db      (13*MAXIMUMFILES) dup (0)

stufft1     db      0, 0
            db      1, 0, 3, 0, 7, 0, 0fh, 0, 01fh
            db      0, 03fh, 0, 07fh, 0, 0ffh, 0
            db      0ffh, 1, 0ffh, 3, 0ffh, 7, 0ffh, 0fh

onergbcount dw      100h                    ; = 256
palettecount dw     300h                    ; = 768
bank_count  dw      0

StartTable  dw      48h, 24h, 12h, 8
IncCount    db      1                       ; 1 or 8, then 1, 8, 12h, 24h, 48h
;

abs_screen_width dw 200h                    ;   = ScreenWidth
VideoMode   dw      0
VideoMode2  dw      0
screenwidth dw      0
;--------------------------------------------------
;--- GIF Header
headerofGIF db      0, 0, 0, 0, 0, 0        ; 6
GIF_width   dw      0                       ; 2
GIF_height  dw      0                       ; 2
rgbstuff    db      0, 0, 0                 ; 3
;--- Global palette
header_palet db     0                       ; 768 = 766+2       first byte
            db      766 dup (0)
;            times      766 db 0
headlastb_pal db    0                       ;                   last byte
;--- Image block
imgblk_left   dw    0
imgblk_top    dw    0
imgblk_width  dw    0
imgblk_depth  dw    0                       ; not used
imgblk_flags  db    0
;--- local palette
firstpalbyte db     767 dup (0)             ;first palette byte --- 768-1=767
;firstpalbyte times 767 db 0
lastpalbyte db      0                       ;last palette byte
;---
byte_0_870  db      0
            db      0
            db      0
neverused   dw      0
;---------------------------------
segalloc2   dw      0
segalloc1   dw      0
filecounter dw      0
Offst_Source dw     0
GIFfilehandle dw    0

;
; Bunch of temp. variables
;
byte_0_87f  db      0
byte_0_880  db      0
byte_0_881  db      0
byte_0_882  db      0
word_0_883  dw      0
word_0_885  dw      0
word_0_887  dw      0
word_0_889  dw      0
word_0_88b  dw      0
word_0_88d  dw      0
destin      dw      0
position    dw      0
word_0_893  dw      0
word_0_895  dw      0
word_0_897  dw      0
widthbound  dw      0
;------------
flagtheext  db      0
flagmodetxt db      YES
screenheight dw     0
;------------
readblock:
            cmp     word ptr[filecounter], 0          ; 0
            jz      readfromfile

fromfiletoalloc1:
            push    si
            push    ds
            mov     si, [Offst_Source]
            mov     ax, [segalloc1]
            mov     ds, ax
            lodsb
            pop     ds
            mov     [Offst_Source], si
            dec     word ptr[filecounter]
            pop     si
            ret
;------------
readfromfile:
            push    bx
            push    cx
            push    dx
            push    ds
            mov     bx, [GIFfilehandle]
            mov     cx, 0ffffh              ; bytes to read = 65535 = 1 bank
            xor     dx, dx
            mov     ax, [segalloc1]
            mov     ds, ax
            mov     ah, 3fh
            int     21h                     ; DOS - 2+ - READ FROM FILE WITH HANDLE
                                            ; BX = file handle = GIFFileHandle
                                            ; CX = number of bytes to read
                                            ; DS:DX to buffer = SegAlloc1
            pop     ds
            jnb     readwasok

readerror:
            mov     dx, offset file_read_error ; 'File read error.$'
            jmp     endwitherror
;------------
readwasok:
            or      ax, ax
            jz      readerror
            mov     word ptr[filecounter], ax
            mov     word ptr[Offst_Source], 0
            pop     dx
            pop     cx
            pop     bx
            jmp short     fromfiletoalloc1
;------------
subreading:
            cmp     byte ptr[byte_0_87f], 0
            jnz     loc_0_8f7
            call    readblock
            mov     [byte_0_87f], al

loc_0_8f7:
            call    readblock
            dec     byte ptr[byte_0_87f]
            mov     [byte_0_881], al
            ret
;------------
topsubreading:
            cmp     byte ptr[byte_0_880], 0
            jz      loc_0_941

loc_0_909:
            mov     bl, [byte_0_881]
            mov     bh, 0
            mov     cl, 8
            sub     cl, [byte_0_880]
            shr     bx, cl

loc_0_917:
            mov     ah, [byte_0_880]
            cmp     ah, [byte_0_882]
            jnb     loc_0_935
            call    subreading
            mov     ah, 0
            mov     cl, [byte_0_880]
            shl     ax, cl
            or      bx, ax
            add     byte ptr[byte_0_880], 8
            jmp short     loc_0_917
;------------
loc_0_935:
            mov     al, [byte_0_882]
            sub     [byte_0_880], al
            and     bx, [word_0_883]
            ret
;------------
loc_0_941:
            call    subreading
            mov     byte ptr[byte_0_880], 8
            jmp short     loc_0_909
;------------
processsegalloc2:
            push    ax
            push    cx
            mov     cl, [byte_0_870]
            mov     [byte_0_882], cl
            inc     byte ptr[byte_0_882]
            mov     ax, 1
            shl     ax, cl
            mov     [word_0_885], ax
            shl     ax, 1
            mov     [word_0_889], ax
            mov     ax, [word_0_885]
            inc     ax
            mov     [word_0_887], ax
            inc     ax
            mov     [word_0_88b], ax
            mov     [word_0_88d], ax
            mov     al, [byte_0_882]
            mov     ah, 0
            shl     ax, 1
            mov     di, offset stufft1
            add     di, ax
            mov     ax, [di]
            mov     [word_0_883], ax
            pop     cx
            pop     ax
            ret
;------------
; pgm data
;------------
; DOS 8.3
FILELEN     EQU       13
LineCounter dw      0
            dw      0
alphashift  dw      0                       ; This is the Horiz. centering shift
betashift   dw      0                       ; This is the Vertical centering shift
wildstar    db      '*.GIF', 0
flagfirst   db      0
;------------
noGIFfound:
KgifTitle   db      0Ah, 0Dh
            db      '=====================================', 0DH, 0AH
            db      '==       KGIF version '
            db      major_version, '.', minor_version
            db                               '          ==', 0DH, 0AH
            db      '== by Benny_land@geocities.com     ==', 0dh, 0ah
            db      '=====================================', 0DH, 0AH
            db      0DH, 0AH
            db      'No GIFs Found.', 0dh, 0ah, 0dh, 0ah
            db      'Usage: "KGIF GIF_file[.GIF]"', 0dh, 0ah
            db      'or "KGIF" in a directory with GIFs.'
            db      '$'

;------------
zoom        db      0
shiftzoom   db      0
vgapic      db      0
file_pos    dw      0
okflag      db      0
GIFfilecounter dw   0                       ; Number of files to sort.
Card        dw      1
Cardshift   dw      0

;------------
; top of Bank_Switching_Routine
;
di_is_null:
            mov     di, [cs: LineCounter]   ; di = [LineCounter]
            add     di, [cs: imgblk_top]    ; di = di + [imgblk_top]
            add     di, [cs: BetaShift]     ; Shifts pic down 
; di = y
            mov     bx, ax
            push    ax
            mov     ax, [cs: ScreenWidth]   ; ax = Screen Width
            mul     di                      ; ax = width * y
            add     bx, [cs: imgblk_left]   ; bx = bx + [imgblk_left]
            add     bx, [cs: AlphaShift]    ; Shifts pic left 
; bx = x
            add     ax, bx                  ; ax = width*y + x (ptr)
            adc     dx, 0                   ; dx = 1:0000h (bank)
; dx = bank
            mov     di, ax                  ; di = ptr
; di = [x][y]

; Are we in 320x200?
            ;cmp     word ptr[cs: VideoMode], 13h
            cs cmp     word[VideoMode], 13h
            je      windows_end      ; else crash if directbank is called

; bank *= winsize/wingran;
            push    cx
            mov     cx,word ptr[cs: Cardshift]
            shl     dx,cl
            pop     cx

;            mov     ax,dx
;            cwd
;            mov     bx,[cs: Card]
;            mul     bx
;            mov     dx,ax

; comment this line out if you want direct banking:
            jmp     VESAbankswitch
;-------
; Direct bank switch
;-------
                                        ; Window A: exists and is writable?
            test    byte ptr[cs: window_attributes_window_A], 5 ; 101b
            jnz     windowB   ; bits not set
            xor     bx, bx

; Direct bank switch:
;            call    dword ptr[cs: window_bank_switch]
            call    far [cs: window_bank_switch]

            jmp short     windows_end

WindowB:                                ; Window B: exists and is writable?
            test    byte ptr[cs: window_attributes_window_B], 5
            jnz     windows_end  ; bits not set

            mov     bx, 1

; Direct bank switch:
;            call    dword ptr[cs: window_bank_switch]
            call    far [cs: window_bank_switch]

            jmp     windows_end

;-------
; Vesa bank switch
;
; Note: on a Tseng video card, it's probably better (mandatory?)
; to skip the tests and do the two Int 10h calls (window A + B)
;-------
VESAbankswitch:
                                        ; Window A: exists and is writable?
            test    byte ptr[cs: window_attributes_window_A], 5 ; 101b
            jnz     windowB_   ; bits not set
            xor     bx, bx

; VESA bank switch:
            mov     ax, 4f05h
            Int     10h

            jmp short     windows_end

WindowB_:                               ; Window B: exists and is writable?
            test    byte ptr[cs: window_attributes_window_B], 5
            jnz     windows_end  ; bits not set

            mov     bx, 1

; VESA bank switch:
            mov     ax, 4f05h
            Int     10h
                             ; DX = Bank Number.
                             ; VESA SuperVGA BIOS - CPU VIDEO MEMORY CONTROL
                             ; BL = 00h window A, 01h window B
                             ; Return: AL = 4Fh function supported
                             ; AH = 00h successful, 01h failed
                             ; BH = subfunctionselect video memory window
windows_end:

            pop     ax
            jmp short     blittoscreen
;------------
; We come here from decode:
;------------
Bank_Switching_Routine:

            pusha

            mov     ax, [LineCounter]
            add     ax, [BetaShift]         ; Shifts pic down 

            cmp     ax, [ScreenHeight]
            jb      s_inrange

;
            inc     word ptr[LineCounter]

            mov     ax, [imgblk_width]      ; [WidthBound] = GIF width
            mov     [WidthBound], ax        ; restore decoder ctr
            popa
            mov     di, 32768
            ret
s_inrange:

            push    es
            push    ds
;------------
            mov     ax, 0a000h              ; video address
            mov     es, ax                  ;  into ES
;------------
            mov     ax, [cs: SegAlloc2]     ; seg source
            mov     ds, ax                  ;  into DS:
;------------
            mov     cx, 1
calculatelinelen:
            push    cx                      ; cx = 1
            mov     cx, [cs: imgblk_width]
            cmp     cx, [cs: ScreenWidth]
            jb      keep_imgblk_width
            mov     cx, [cs: ScreenWidth]
keep_imgblk_width:
            xor     ax, ax
            mov     si, 32768
            jmp     di_is_null
;------------
check_if_di_is_null:
            or      di, di                  ; di overflowed? (0FFFFh+1==0)
            jnz     blittoscreen
            jmp     di_is_null

blittoscreen:
; Count colors
            cmp     byte ptr[cs: ToggleCount], YES
            jne     noToggleCount
            xor     bx, bx
;            mov     bl, byte ptr ds:[si]
            ds mov     bl, byte ptr [si]
            mov     byte ptr[cs: colorcount+bx], 1   ; flag our color counter
noToggleCount:

; Blit to screen:
            movsb                           ; ES:[DI] <= DS:[SI]; si++; di++; ES = A000h
            inc     ax
            loop    check_if_di_is_null
;------------

            mov     cl, [cs: IncCount]
            mov     ch, 0                   ; CL = IncCount; 1, 8, 18, 36, 72
            add     [cs: LineCounter], cx   ; LineCounter += IncCount;

            mov     cx, [cs: GIF_Height]
            cmp     [cs: LineCounter], cx   ; if (LineCounter < GIF_Height)
            jb      moretogo                ;   goto moretogo;

            mov     bx, [cs: Bank_Count]
            shl     bx, 1                   ; bx *= 2;
            ;mov     ax, cs:[bx+offset StartTable] ; 0048 0024 0012 0008
            cs mov     ax, [bx+offset StartTable] ; 0048 0024 0012 0008
            mov     [cs: IncCount], ah
            cbw
            mov     [cs: LineCounter], ax
            inc     word ptr[cs: Bank_Count]
moretogo:
            pop     cx                      ; get back 1
            loop    calculatelinelen        ; 1 loop to cx = 0
;------------
lineofpixeldone:
            pop     ds
            pop     es
            mov     ax, [cs: imgblk_width]
            mov     [cs: WidthBound], ax
            popa
            mov     di, 32768
            ret
;-------------------------------------------------------------
writepalette:
            mov     ax, 1012h
            xor     bx, bx
            mov     cx, [cs: OneRGBcount]
            mov     dx, si
            int     10h
                                            ; - VIDEO - SET BLOCK OF DAC REGISTERS (EGA, VGA/MCGA)
                                            ; BX = starting color register, CX = number of registers to set
                                            ; ES:DX to table of 3*CX bytes where each 3 byte group represents one
                                            ; byte each of red, green and       blue (0-63)
            ret
;------------
; get next image block
Get_Next_Image_Block:
            mov     word ptr[LineCounter], 0
            mov     word ptr[Bank_Count], 0

            mov     di, offset imgblk_left
            mov     cx, 9
loop_readblock:
            call    readblock
            stosb
            loop    loop_readblock

            mov     si, offset header_palet ; ref 1     (first)
            call    writepalette

            test    byte ptr[imgblk_flags], 40h ; interlaced?
            mov     byte ptr[IncCount], 1
            jz      imgblk_is_not_interlaced

            mov     byte ptr[IncCount], 8

imgblk_is_not_interlaced:
            test    byte ptr[imgblk_flags], 80h ; local colour map?
            jz      processGIFdata              ; no

            mov     di, offset firstpalbyte
            test    byte ptr[imgblk_flags], 8
            jz      thecolorsare256         ;/[imgblk_flags]_Is_8

            std                             ; set direction flag for decrementing
            mov     di, offset lastpalbyte

thecolorsare256:
            mov     cx, 768                 ;= Number of RGB Colors

loop_readblock2:
            call    readblock
            stosb
            loop    loop_readblock2

            cld
            mov     si, offset firstpalbyte
            call    writepalette

processGIFdata:
            call    readblock
            mov     [byte_0_870], al
            mov     word ptr[Destin], 8000h           ; = 32768
            mov     word ptr[Position], 0
            mov     byte ptr[byte_0_880], 0
            mov     byte ptr[byte_0_87f], 0
            mov     ax, [cs: SegAlloc2]
            mov     es, ax
            call    ProcessSegAlloc2
            mov     ax, [imgblk_width]
            mov     [WidthBound], ax
            xor     ax, ax
            mov     [NeverUsed], ax

decode1_abb:
            call    topsubreading
            cmp     bx, [word_0_887]
            jnz     decode2_ac7
            jmp     resetallsegments
;------------
decode2_ac7:
            cmp     bx, [word_0_885]
            ja      decode3_ad7
            jz      decode5
            mov     bl, [bx+offset decoder_t2] ; 256 bytes
            mov     bh, 0

decode3_ad7:
            mov     [word_0_893], bx
            mov     di, 0
            cmp     bx, [word_0_88d]
            jb      decode4_aea
            mov     ax, [word_0_897]
            stosb
            mov     bx, [word_0_895]

decode4_aea:
            cmp     bx, [word_0_885]
            jl      decode7
            mov     si, 2000h               ; =8192
            ;mov     al, es:[ bx+si]
            es mov     al, [ bx+si]
            stosb
            mov     si, 4000h               ; =16, 384
            shl     bx, 1
            ;mov     bx, es:[ bx+si]
            es mov     bx, [ bx+si]
            jmp short     decode4_aea
;------------
decode5:
            call    ProcessSegAlloc2

decode6:
            call    topsubreading
            cmp     bx, [word_0_885]
            jz      decode6
            mov     bl, [bx+offset decoder_t2] ; 256 bytes
            mov     bh, 0
            mov     [word_0_895], bx
            mov     [word_0_897], bx
            mov     di, [Destin]
            mov     al, bl
            stosb

            mov     cx, 1
            jmp     decode_11
;------------
decode7:
            mov     al, bl
            stosb

            mov     [Position], di
            mov     ax, bx
            mov     bx, [word_0_88d]
            mov     [word_0_897], ax
            mov     di, 2000h
            ;mov     es:[ bx+di], al
            es mov     [ bx+di], al
            mov     di, 4000h
            mov     ax, [word_0_895]
            shl     bx, 1
            ;mov     es:[ bx+di], ax
            es mov     [ bx+di], ax
            inc     word ptr[word_0_88d]
            mov     ax, [word_0_893]
            mov     [word_0_895], ax
            mov     bx, [word_0_88d]
            cmp     bx, [word_0_889]
            jb      decode8
            mov     al, [byte_0_882]
            cmp     al, 0ch
            jnb     decode8
            shl     word ptr[word_0_889], 1
            shl     word ptr[word_0_883], 1
            inc     word ptr[word_0_883]
            inc     byte ptr[byte_0_882]

decode8:
            mov     si, [Position]
            mov     di, [Destin]
            mov     cx, si
            sub     cx, 0
            inc     si                      ; Position++
            cmp     cx, [WidthBound]
            jnb     decode_10               ; if (Position >= [WidthBound])
            sub     [WidthBound], cx

decode9:
            push    ds
            push    ax
            mov     ax, [cs: SegAlloc2]     ; src
            mov     ds, ax
            pop     ax
            sub     si, 2
            movsb                           ; ES:[DI] <= DS:[SI]; si++; di++;

            pop     ds
            loop    decode9
            mov     [Destin], di
            jmp     decode1_abb
;------------
decode_10:
            sub     si, 2
            push    ds
            push    ax
            mov     ax, [cs: SegAlloc2]     ; src
            mov     ds, ax
            pop     ax
            movsb                           ; ES:[DI] <= DS:[SI]; si++; di++;

            pop     ds

decode_11:
            dec     word ptr[WidthBound]                ; Decrease
            jnz     decode_12                           ; Is it zero?
            call    Bank_Switching_Routine              ;   Yes, switch

decode_12:
            loop    decode_10
            mov     [Destin], di
            jmp     decode1_abb
;------------
resetallsegments:
            mov     ax, cs
            mov     ds, ax
            mov     es, ax
            ret
;------------
; data
;------------
olddi       dw      0
onefileonly db      0
;
; Help splash screen
;
AllTheCommands  db      "           "
                db      "KGIF ", major_version, '.', minor_version
                db      "                ", 0Dh, 0Ah
                db      "        -= Commands =-             ", 0Dh, 0Ah
                db      "                                   ", 0Dh, 0Ah
                db      " Esc:          Quit                ", 0Dh, 0Ah
                db      " i, ->:        Info                ", 0Dh, 0Ah
                db      " p, <-:        Palette             ", 0Dh, 0Ah
                db      " +-:           Zoom in/out         ", 0Dh, 0Ah
                db      " /:            Toggle centering    ", 0Dh, 0Ah
                db      " 0:            Toggle display mode ", 0Dh, 0Ah
                db      " Dollar:       Toggle count colors ", 0Dh, 0Ah
                db      " End:          Last                ", 0Dh, 0Ah
                db      " Enter:        Refresh             ", 0Dh, 0Ah
                db      " Home:         First               ", 0Dh, 0Ah
                db      " PgDn:         Down 10             ", 0Dh, 0Ah
                db      " PgUp:         Up 10               ", 0Dh, 0Ah
                db      " Space:        Restore palette     ", 0Dh, 0Ah
                db      " Up arrow:     Up                  ", 0Dh, 0Ah
                db      " g:            Grayscale           ", 0Dh, 0Ah
                db      " t:            Trash palette       ", 0Dh, 0Ah
                db      " Other keys:   Next GIF.           ", 0Dh, 0Ah
                db      "                                   ", 0Dh, 0Ah
                db      "$"

;%-----------------------------------------------%
;               Entry Point
;%-----------------------------------------------%
Begin_The_Show:
            cld
            cli
            push    cs
            pop     ss

; more here?
            mov     sp, offset finalbyte+202
            sti

            mov     ah, 4ah

; How much to allocate for this proggie?
            mov     bx, 750
                             ; 750*16 = 12,000   (too much/too little?)
                             ; 700*16 = 11,200
                             ; 600*16 =  9,600
                             ; paragraphs (16 * xxx)

            int     21h      ; DOS - 2+ - ADJUST MEMORY BLOCK SIZE (SETBLOCK)
                             ; ES = segment address of block to  change
                             ; BX = new  size in paragraphs
            mov     ax, cs
            mov     es, ax
            mov     ds, ax
            cmp     byte ptr[cs: 80h], 0
            jnz     get_file_name
            jmp short     findfirst
;--------------
; SINGLE FILE MODE
;--------------
get_file_name:
            mov     word ptr[GIFFileCounter], 1
            mov     si, 81h                 ; source filename
            mov     di, offset filestore

storefilename:
            lodsb
            cmp     al, "."
            jne     noextension
            mov     byte ptr[flagtheext], 1
noextension:
            cmp     al, 0dh
            jz      end_of_file_name
            cmp     al, 00h
            jz      end_of_file_name
            cmp     al, 20h
            jz      storefilename
            stosb

            jmp short     storefilename
;------------
end_of_file_name:
            cmp     byte ptr[flagtheext], 1
            je      extensionfound
            mov     al, "."
            stosb
            mov     al, "g"
            stosb
            mov     al, "i"
            stosb
            mov     al, "f"
            stosb
extensionfound:
            xor     al, al
            stosb                           ; "\0" at end of filename.
            mov     byte ptr[OneFileOnly], 1
            jmp short     jumpone

;--------------
; CAROUSEL MODE
;--------------
findfirst:
            mov     ah, 4eh
            mov     di, offset filestore
find_next:
            mov     al, 0
            mov     dx, offset wildstar
            mov     cx, 20h
            int     21h                     ; DOS func ( ah ) = 4Eh
                                            ; FIND FIRST: Start file search
                                            ;CX-attr to search on
                                            ; DS:DX-ASCIIZ string
                                            ;if CF=1 AX-ret code
            jb      terminate               ; Jump if below     ( < )
            inc     word ptr[GIFFileCounter]
            cmp     word ptr[GIFFileCounter], MAXIMUMFILES
            ja      terminate

            mov     si, FILENAME            ; ?? better way?
            mov     cx, 13
            rep     movsb

            mov     ah, 4fh                 ; find next
            jmp short     find_next
terminate:
            cmp     word ptr[GIFFileCounter], 0        ; files found?
            jne     sortandlist
            mov     dx, offset noGIFfound
            jmp     endwitherror

;       Sort Files
sortandlist:
            mov     [OldDI], di             ; not used anymore

            push    bx                      ; Push a bunch of registers
            push    dx
            push    si
            push    ds

            push    cs                      ; Push CS...
            pop     ds                      ;        into DS.

            mov     di, offset filestore    ; This is the beginning.
            mov     cx, [GIFFileCounter]    ; Number of files to sort.
            dec     cx                      ; Loop needs one less than that.
            jcxz    allsorted               ; But zero means only one file.

sortloop1:  push    cx                      ; Save the file counter.
            mov     si, di                  ; Set source to destination.

sortloop2:  add     si, FILELEN             ; Set source to next file.
            push    cx                      ; Save the counter,
            push    si                      ;    compare source,
            push    di                      ;    and compare destination.
            mov     cx, FILELEN             ; x characters to compare.

            repz    cmpsb                   ; Do the compare.
                                            ; cmp es:[di] from ds:[si], inc si & di.

            jae     noswitch                ; Jump if already in order.
            pop     di                      ; Get back these registers...
            pop     si

            push    si                      ; And push them again for move.
            push    di

; /* SWAP es:[di], ds:[si] */
            mov     cx, FILELEN             ; x characters.
switchloop:
            ;mov     al, es:[ di]           ; Character from destination...
            es mov     al, byte[di]         ; Character from destination...
;           mov     al,es:[di]

            movsb                           ; Source to destination...

            ;mov     ds:[ si - 1], al       ; Character to source...
            ds mov     byte[si - 1], al     ; Character to source...

            loop    switchloop              ; For the rest of the line.

noswitch:   pop     di                      ; Get back the registers...
            pop     si
            pop     cx
            loop    sortloop2               ; And loop for next file.

            pop     cx                      ; Get back file counter...
            add     di, FILELEN             ; Compare with next file...
            loop    sortloop1               ; And loop again.
allsorted:
;--------------
; Now we got our file name(s)
;--------------
jumpone:
            mov     ax, cs
            mov     es, ax
            mov     ds, ax

            mov     si, offset filestore     ; source for filename 1
            mov     word ptr[File_Pos], 0

;--------------
; We branch here from down: (Menu)
;--------------
store_para_list:

            xor     ax, ax

; count colors
            cmp     byte ptr[ToggleCount], YES
            jne     noToggleCount2
            mov     di, offset colorcount
            mov     cx, 256
            rep     stosb                   ; zeroes our table
noToggleCount2:
            mov     di, offset paramlist    ; target for filename
store_file_name:
            mov     cx, 13
            cmp     byte ptr[OneFileOnly], 1
            jne     isshorter
            mov     cx, 256     ; was 40
isshorter:
            rep     movsb

;--------------
; Skip memory alloc if already done
;--------------
            cmp     byte ptr[FlagFirst], 1
            je      already_done
            mov     byte ptr[FlagFirst], 1

gotfilename:
            xor     al, al
            mov     di, offset decoder_t2   ; 256 bytes
            mov     cx, 256
buildtable:
            stosb
            inc     al                      ; 0, 1, .. , 255.
            loop    buildtable

            mov     ah, 48h
            mov     bx, 0a01h               ;=2561  x 16= 40, 976
            int     21h                     ; DOS - 2+ - ALLOCATE MEMORY
                                            ; BX = number of 16-byte paragraphs desired
            jnb     allocsuccess
            mov     dx, offset out_of_memory ; 'Out of memory.$'
            jmp     endwitherror

;------------
allocsuccess:
            mov     [SegAlloc2], ax         ; [SegAlloc2] = malloc(40976)

            mov     ah, 48h
            mov     bx, 1000h               ; =4096 x 16= 65536
            int     21h                     ; DOS - 2+ - ALLOCATE MEMORY
                                            ; BX = number of 16-byte paragraphs desired
            jnb     openGIFfile
            mov     dx, offset out_of_memory ; 'Out of memory.$'
            jmp     endwitherror

;------- data ------------
segalloc3   dw      0
;-------------------------

allocseg3:
            mov     ah, 48h
            mov     bx, 4642                ; x 16= 65000
            int     21h                     ; DOS - 2+ - ALLOCATE MEMORY
                                            ; BX = number of 16-byte paragraphs desired
            jnb     goback
            mov     dx, offset out_of_memory ; 'Out of memory.$'
            jmp     endwitherror
goback:
            mov     [SegAlloc3], ax           ; [SegAlloc3] = malloc(65000)
            ret
freealloc3:
            push    es
            mov     ax, [cs: SegAlloc3]
            mov     es, ax
            mov     ah, 49h
            int     21h                     ; DOS - 2+ - FREE MEMORY
                                            ; ES = segment address of area to be freed
            pop     es
            ret
;------------
openGIFfile:
            mov     [SegAlloc1], ax           ; SegAlloc1 = malloc(65536)

            push    cs
            pop     ds
already_done:
            mov     ax, 3d00h
            mov     dx, offset paramlist
            int     21h                     ; DOS - 2+ - OPEN DISK FILE WITH HANDLE
                                            ; DS:DX to ASCIZ filename
                                            ; AL = access mode
                                            ; 0 - read
            jnb     initreadGIF             ; ok

;            call    displaylastGIFname
            mov     dx, offset file_not_found ; 'File not found.$'
            jmp     endwitherror

;------------
initreadGIF:
            mov     [GIFFileHandle], ax
            mov     di, offset headerofGIF
            mov     cx, 0dh                 ; = 13

readGIFheader:
            call    readblock
            stosb
            loop    readGIFheader
;------------
; Check if it is a GIF87a/89a
;------------
            cmp     byte[headerofGIF+0],'G'
            jne     notagif
            cmp     byte[headerofGIF+1],'I'
            jne     notagif
            cmp     byte[headerofGIF+2],'F'
            jne     notagif
            cmp     byte[headerofGIF+3],'8'
            jne     notagif
            cmp     byte[headerofGIF+4],'7'
            je      itsagif87
            cmp     byte[headerofGIF+4],'9'
            jne     notagif
itsagif87:
            cmp     byte[headerofGIF+5],'a'
            jne     notagif
            jmp     Findvideomode
notagif:
            mov     dx,offset Not_a_GIF_file
            jmp     endwitherror

;------------
; Find video mode
;------------
Findvideomode:
            mov     al,byte ptr[ToggleCenter2]
            mov     byte ptr[ToggleCenter],al          ; get back user choice

            mov     ax, [GIF_Width]
            mov     cx, [GIF_Height]

            cmp     byte ptr[Zoom], 1
            je      mode320x200x256
            cmp     byte ptr[Zoom], 2
            je      mode640x480x256
            cmp     byte ptr[Zoom], 3
            je      mode800x600x256
            cmp     byte ptr[Zoom], 4
            je      mode1024x768x256
nozoom:
            mov     byte ptr[VGApic], 0
            cmp     ax, 320
            ja      l1
            cmp     cx, 200
            ja      l1
            mov     byte ptr[VGApic], 1
mode320x200x256:
            ;mov     word ptr[VideoMode], 13h     ;   VGA MODE - 320x200x256
            mov     word[VideoMode], 13h          ;   VGA MODE - 320x200x256
            mov     word ptr[ScreenWidth], 320
            mov     word ptr[ScreenHeight], 200
            jmp short     nextstep
l1:
            cmp     ax, 640
            ja      l2
            cmp     cx, 480
            ja      l2
mode640x480x256:
            ;mov     word ptr[VideoMode], 101h    ; VESA MODE - 640x480x256
            mov     word[VideoMode], 101h         ; VESA MODE - 640x480x256
            mov     word ptr[ScreenWidth], 640
            mov     word ptr[ScreenHeight], 480
            jmp short     nextstep
l2:
            cmp     ax, 800
            ja      l3
            cmp     cx, 600
            ja      l3
mode800x600x256:
            ;mov     word ptr[VideoMode], 103h    ; VESA MODE - 800x600x256
            mov     word[VideoMode], 103h         ; VESA MODE - 800x600x256
            mov     word ptr[ScreenWidth], 800
            mov     word ptr[ScreenHeight], 600
            jmp short     nextstep
l3:
            cmp     ax, 1024
            ja      l4
            cmp     cx, 768
            ja      l4
            jmp short     mode1024x768x256
l4:
            mov     byte ptr[cs: ToggleCenter], NO ; too big pic - centering off

mode1024x768x256:
            ;mov     word ptr[VideoMode], 105h    ; VESA MODE - 1024x768x256
            mov     word[VideoMode], 105h         ; VESA MODE - 1024x768x256
            mov     word ptr[ScreenWidth], 1024
            mov     word ptr[ScreenHeight], 768
            jmp short     nextstep

;------------
nextstep:
            cmp     byte ptr[cs: Zoom], 0
            jne     inazoom
            mov     ax, word ptr[cs: VideoMode]
            mov     word ptr[cs: VideoMode2], ax
inazoom:
notsamevidmode:

            mov     word ptr[cs: AlphaShift], 0
            mov     word ptr[cs: BetaShift], 0
            cmp     byte ptr[cs: ToggleCenter], NO      ; centering on?
            je      nextnextstep                        ; no, skip

            mov     ax, word ptr[cs: VideoMode]
            cmp     word ptr[cs: VideoMode2],ax
            jb      greatervidmode              ; old mode > new mode
            ;cmp     word ptr[cs: GIF_Width],1024
            cs cmp     word[GIF_Width],1024
            ja      nextnextstep
            ;cmp     word ptr[cs: GIF_Height],768
            cs cmp     word[GIF_Height],768
            ja      nextnextstep

            cmp     word ptr[cs: VideoMode2],ax
            je      greatervidmode              ; old mode == new mode
            cmp     byte ptr[cs: Zoom], 0       ; are we in a zoom?
            jne     nextnextstep                ; centered zoom is buggy - skip!
greatervidmode:
; Center pic horiz:
            mov     ax, [cs: ScreenWidth]
            mov     bx, [cs: GIF_Width]
            sub     ax, bx
            shr     ax, 1
            mov     word ptr[cs: AlphaShift], ax
; Center pic vertic:
            mov     ax, [cs: ScreenHeight]
            mov     bx, [cs: GIF_Height]
            sub     ax, bx
            shr     ax, 1
            mov     [cs: BetaShift], ax
;--------------------------------------------
nextnextstep:                               ; <- jump here for no centering of the pic
            mov     word ptr[OneRGBcount], 0
            test    byte ptr[RGBstuff], 80h
            jz      donotprocessthergb
            mov     cl, [RGBstuff]
            and     cl, 7
            mov     ax, 2
            shl     ax, cl
            mov     word ptr[OneRGBcount], ax
            mov     cx, 3                   ;
            mul     cx                      ; ax *= 3;
            mov     [PaletteCount], ax
donotprocessthergb:
            mov     di, offset header_palet ; ref 2     (last)
            test    byte ptr[RGBstuff], 8
            jz      storepalettecountincx
            std                             ; set direction flag for decrementing
            mov     di, offset headlastb_pal ;last byte

storepalettecountincx:
            mov     cx, [PaletteCount]

shiftblockreadby_2:
            call    readblock
            shr     al, 1
            shr     al, 1
            stosb
            loop    shiftblockreadby_2

            cld
            mov     ax, 1a00h
            int     10h                     ; - VIDEO - DISPLAY COMBINATION
                                            ; (PS, VGA/MCGA): read display combination code
            cmp     al, 1ah
            jnz     errordetectingvga
            cmp     bl, 7                   ; vga w/ mono analog display
            jz      letssetthemode
            cmp     bl, 8                   ; vga w/ color analog display
            jz      letssetthemode          ; ok

errordetectingvga:
            mov     dx, offset vga_not_found ; 'VGA not found$'
            jmp     endwitherror
;------------
TurnedOff   db      NO
turnoffscreen:
            cmp     byte ptr[ToggleTurn], YES
            je      turnitoff
            ret
turnitoff:
            cmp     byte ptr[TurnedOff], YES
            je      turnitoffdone
            mov     dx, 03c4h
            mov     al, 1
            out     dx, al
            inc     dx
            in      al, dx
            or      al, 020h
            out     dx, al
            mov     byte ptr[TurnedOff], YES
turnitoffdone:
            ret

turnonscreen:
            cmp     byte ptr[ToggleTurn], YES
            je      turniton
            ret
turniton:
            cmp     byte ptr[TurnedOff], NO
            je      turnitondone
            mov     dx, 03c4h
            mov     al, 1
            out     dx, al
            inc     dx
            in      al, dx
            and     al, 0dfh
            out     dx, al
            mov     byte ptr[TurnedOff], NO
turnitondone:
            ret
;------------
retrace:
            mov     dx, 3DAh
            in      al, dx
            test    al, 8       ; sync while CTR is off
            jnz     retrace     ; bit set
retrace2:
            in      al, dx
            test    al, 8       ; sync while CTR is on
            jz     retrace2     ; bit not set
            ret
;------------
; set mode
;------------
letssetthemode:
            call    turnoffscreen           ; 1x

            mov     bx, word ptr[VideoMode] ; get video mode

            mov     cx, [ScreenWidth]
            mov     [Abs_Screen_Width], cx

            cmp     bx, 13h                 ; 13h?
            jnz     setVESAmode             ; nope, jump

            call    retrace

            mov     ax, bx                  ; yep, use it as a call = 0013h
            int     10h                     ; - VIDEO - 320x200x256 VIDEO MODE

            mov     byte ptr[FlagModeTxt], NO
            jmp short     readthefilemaindata
;------------
; - SET VIDEO MODE - VESA -
setVESAmode:
            call    GetSVGAModeinfo
            call    retrace

            mov     byte ptr[FlagModeTxt], NO

            mov     ax, 4f02h
            int     10h                     ; - VIDEO - VESA SuperVGA BIOS -
                                            ; SET SuperVGA VIDEO MODE
                                            ; BX = mode
                                            ; BX = bit 15 set means don't clear video memory
                                            ; Return: AL = 4Fh function supported
                                            ; AH = 00h successful, 01h failed
            cmp     ax, 4fh
            jz      readthefilemaindata

            mov     dx, offset VESA_not_found ; 'VESA not found$'
            jmp     endwitherror
;------------
readthefilemaindata:

            call    turnoffscreen           ; 2x

            call    readblock
            or      al, al
            jz      readthefilemaindata

            cmp     al, ';'                 ; 59 = 3Bh = ';' to end of GIF
            jnz     checkextra1
            jmp short     closeGIFfile
;------------
; buggy if too much garbage after the end of the GIF?
;
checkextra1:
            cmp     al, '!'                 ; 33 = 21h = '!'
            jnz     checkextra2
;--------------------------------------------------------
; New code:
; "it got into a loop if a Graphics Control Extension Block
; contained zero as data." (fix by olavi.ostman@tietotehdas.fi)
;------- changed from here -----------------------------
	call	readblock		;skip Graphic Control Label
	call	readblock		;get Block Size
	mov	ah, 0
	mov	cx, ax
loophere:
	    call    readblock
;	    or      al, al	;does not work if data = 00h
;	    jnz     loophere
	loop	loophere		;skip the Block
;------- to here ----------------------------------------
; Old code:
;
;loophere:
;            call    readblock
;            or      al, al
;            jnz     loophere
;--------------------------------------------------------
            jmp short     readthefilemaindata
;------------
; image block?
checkextra2:
            cmp     al, ','                 ; 44 = 2C = ','
            jz      imgblockfound
            mov     dx, offset img_block_not_found
            jmp     endwitherror        ; go next one instead?
;------------
imgblockfound:
            call    Get_Next_Image_Block
            jmp short     readthefilemaindata
;------------
textmode:
            cmp     byte ptr[FlagModeTxt], YES
            je      textmodedone
            call    turnoffscreen
            mov     ax, 3
            int     10h                     ; set Video mode 3
            call    turnonscreen
            mov     byte ptr[FlagModeTxt], YES
textmodedone:
            ret
getkey:
            mov     ah, 0
            int     16h                     ; KEYBOARD  - READ CHAR FROM BUFFER, WAIT IF EMPTY
                                            ; Return: AH = scan code, AL = character
            ret

;------ setcolor        ---------------------------------------------------
setcolor:
            mov     ax, 1010h
            mov     bx, 7
            mov     dh, 0
            mov     ch, 63
            mov     cl, 63
            int     10h                     ; zap the pen color

            mov     ax, 1010h
            mov     bx, 0
            mov     dh, 0
            mov     ch, 0
            mov     cl, 30
            int     10h                     ; zap the background
            ret
;------------------------------
; All decoding done.
;------------------------------
closeGIFfile:
            push    cs
            pop     ds

            call    turnonscreen

            mov     ah, 3eh
            mov     bx, [GIFFileHandle]
            int     21h                     ; DOS - 2+ - CLOSE A FILE WITH HANDLE
                                            ; BX = file handle
;------------------------------
; MENU
;------------------------------
menu:
            call    getkey
            cmp     al, 01bh                 ; Esc?
            jne     carryon
            jmp     byebye
carryon:
;------------------------------
; Show info
;------------------------------
            cmp     AL, 'i'
            je      infoi
            cmp     AH, 4dh           ; '->' arrow?
            je      infoi
            jmp     notinfo
infoi:
            call    setcolor
            mov     ax, [File_Pos]
            mov     bl, 13
            mul     bl                      ; ax = al*byte
            mov     si, offset filestore
            add     si, ax
loopinfofile:
            mov     ah, 2
            mov     dl, byte ptr[si]
            cmp     dl, 0
            je      endoffilename
            int     21h                     ; one char output
            inc     si
            jmp short     loopinfofile

endoffilename:
            mov     ah, 2
            mov     dl, ' '
            int     21h
            mov     ax, word[GIF_Width]
            call    utoa
            mov     ah, 2
            mov     dl, 'x'
            int     21h
            mov     ax, word[GIF_Height]
            call    utoa
            mov     ah, 2
            mov     dl, '-'
            int     21h
            mov     ax, word[OneRGBcount]
            call    utoa
            mov     ah, 2
            mov     dl, 'c'
            int     21h
; COUNTCOLORS
            cmp     byte ptr[ToggleCount], YES
            jne     noToggleCount3
; Let's count how many colors this GIF used:
            mov     ah, 2
            mov     dl, ' '
            int     21h
            mov     ah, 2
            mov     dl, '('
            int     21h
            xor     bx, bx
            mov     cx, 256
            mov     si, offset colorcount
countcol:
            lodsb
            add     bx, ax
            loop    countcol

            mov     ax, bx
            call    utoa                    ; display n of colors used.
            mov     dx, offset infocolorcount
            mov     ah, 9
            int     21h
noToggleCount3:
; EOL:
            mov     ah, 2
            mov     dl, 0dh
            int     21h

            mov     ah, 2
            mov     dl, 0ah
            int     21h

            mov     dx, offset picture_number
            mov     ah,9
            int     21h

            mov     ax, [cs: File_Pos]
            inc     ax
            call    utoa

            mov     dx, offset out_of
            mov     ah,9
            int     21h

            mov     ax, [cs: GIFFileCounter]
            call    utoa

            mov     dx, offset out_of_end
            mov     ah,9
            int     21h

            jmp     menu
;------------------------------
; Zoom function
;------------------------------
notinfo:
            cmp     al, '+'                  ; Plus?
            jne     notplus
            mov     byte ptr[cs: Zoom], 0
            mov     ah, 2
            int     16h
            mov     cl, al                  ; store result in CL

            mov     ax, [File_Pos]
            mov     bl, 13
            mul     bl                      ; ax = al*byte
            mov     si, offset filestore
            add     si, ax

            cmp     word ptr[VideoMode], 105h         ; = 1024x768x256 4
            jne     not4
            mov     byte ptr[Zoom], 3
            jmp     store_para_list
not4:
            cmp     word ptr[VideoMode], 103h         ; = 800x600x256 3
            jne     not3
            mov     byte ptr[Zoom], 2
            jmp     store_para_list
not3:
            cmp     word ptr[VideoMode], 101h         ; = 640x480x256 2
            jne     not2
            mov     byte ptr[Zoom], 1

noshift:
            jmp     store_para_list
not2:                                       ; = 320x200x256 1
            mov     byte ptr[Zoom], 1
            jmp     menu                    ; we are already at maximum
;------------------------------
; Zoom out
;------------------------------
notplus:
            cmp     al, '-'                  ; Minus?
            jne     notmoins

            mov     byte ptr[cs: Zoom], 0

            mov     ax, [File_Pos]
            mov     bl, 13
            mul     bl                      ; ax = al*byte
            mov     si, offset filestore
            add     si, ax

            ;cmp     word ptr[VideoMode], 105h    ; VESA MODE - 1024x768x256 4
            cmp     word[VideoMode], 105h         ; VESA MODE - 1024x768x256 4
            jne     m_not4
            mov     byte ptr[Zoom], 4
            jmp     menu                          ; can't go higher with 1mb
m_not4:
            ;cmp     word ptr[VideoMode], 103h    ; VESA MODE - 800x600x256 3
            cmp     word[VideoMode], 103h         ; VESA MODE - 800x600x256 3
            jne     m_not3
            mov     byte ptr[Zoom], 4
            jmp     store_para_list
m_not3:
            ;cmp     word ptr[VideoMode], 101h    ; VESA MODE - 640x480x256 2
            cmp     word[VideoMode], 101h         ; VESA MODE - 640x480x256 2
            jne     m_not2
            mov     byte ptr[Zoom], 3
            jmp     store_para_list
m_not2:
            ;cmp     word ptr[VideoMode], 13h     ; VGA - 320x200x256 1
            cmp     word[VideoMode], 13h          ; VGA - 320x200x256 1
            jne     m_not1
            mov     byte ptr[Zoom], 2
            jmp     store_para_list
m_not1:
            jmp     menu
;------------------------------
; data
;------------------------------
; Count colors
infocolorcount db   "c used) $"

liney       db      0
colorbuf    db      0
colornum    db      0
squarenum   db      0

;------------------------------
; Display the Palette ([OneRGBcount]=n colors)
;------------------------------
notmoins:
            cmp     AL, 'p'
            je      processpal
            cmp     AL, '*'
            je      processpal
            cmp     AH, 04bh          ; '<-'
            je      processpal
            jmp     nopalproc
processpal:
            mov     si, offset header_palet ; restore if trashed palette
            call    writepalette
            mov     byte ptr[ColorNum], 16            ; how many colors vertically
            mov     byte ptr[SquareNum], 16           ; how many squares per line

            cmp     word ptr[OneRGBcount], 16
            ja      gogogo
            mov     byte ptr[ColorNum], 0
            cmp     word ptr[OneRGBcount], 2
            ja      gogogo
            mov     byte ptr[SquareNum], 2
gogogo:
            mov     al, 0                   ; color
            mov     cx, 0                   ; x
            mov     dx, 0                   ; y
            mov     bh, 0                   ; line of x squares ctr
            mov     bl, 0                   ; x square ctr
            mov     ah, 0                   ; y square ctr
            mov     byte ptr[LineY], 0                ; line of y squares ctr
            mov     byte ptr[ColorBuf], 0             ; buffer for color
dash:
            push    ax
            push    bx
            push    cx
            push    dx
            mov     ah, 0ch
            mov     bh, 0                   ; page
            int     10h                     ; putpixel(cx, dx)
            pop     dx
            pop     cx
            pop     bx
            pop     ax

            inc     cx                      ; next x

            inc     bl                      ; next x in square
            cmp     bl, 20                  ; end of x square?
            jne     dash
; ----- next square:
            mov     bl, 0                   ; reset x square ctr
            inc     al                      ; next color

            inc     bh                      ; x squares ctr
            cmp     bh, [SquareNum]         ; 16 (or 2) x squares already?
            jne     dash                    ; 16*20 = 320
; ----- next line, same square:
            mov     bh, 0                   ; reset line of x squares ctr
            mov     al, [ColorBuf]          ; reset color
            inc     dx                      ; next line
            mov     cx, 0                   ; column 0

            inc     ah                      ; next line in y square
            cmp     ah, 12                  ; 12 lines?
            jne     dash
; ----- next line of squares:
            mov     ah, 0                   ; reset y squ line

            mov     al, [ColorNum]
            or      al, al
            jz      its16
            add     [ColorBuf], al

            mov     al, [ColorBuf]            ; get back color

            inc     byte ptr[LineY]                   ; y squares ctr
            cmp     byte ptr[LineY], 16               ; 16 y squares yet?
            jne     dash
its16:
            jmp     menu
nopalproc:
            cmp     AH, 03bh          ; F1
            je      showcommands
            cmp     AL, 'h'
            je      showcommands
            cmp     AL, '?'
            je      showcommands
            jmp short     notshowcommands
;------------------------------
; Show the commands available:
;------------------------------
showcommands:
            call    gotoxy
            call    setcolor
            mov     dx, offset allthecommands
            mov     ah, 9
            int     21h
            jmp     menu
notshowcommands:
;------------------------------
; GrayScale summing
;------------------------------
            cmp     AL, 'g'
            je      grayscale
            jmp short     notgrayscale
grayscale:
            mov     ax, 101bh
            mov     bx, 0
            mov     cx, 256
            int     10h
            jmp     menu
notgrayscale:
;------------------------------
; Resdisplay same pic:
;------------------------------
            cmp     AL, 0dh
            je      redisplay
            jmp short     dontredisplay
redisplay:
            mov     ax, [File_Pos]
            mov     bl, 13
            mul     bl
            mov     si, offset filestore
            add     si, ax
            jmp     store_para_list
dontredisplay:
;------------------------------
; Toggle centering:
;------------------------------
            cmp     AL, '/'
            jne     dontToggleCenter
            cmp     byte ptr[ToggleCenter2], YES
            jne     settoggle
            mov     byte ptr[ToggleCenter2], NO
            jmp short     redisplay
settoggle:
            mov     byte ptr[ToggleCenter2], YES
            jmp short     redisplay
dontToggleCenter:
;------------------------------
; Toggle turn-on/-off:
;------------------------------
            cmp     AL, '0'
            jne     dontToggleTurn
            cmp     byte ptr[ToggleTurn], YES
            jne     setToggleTurn
            mov     byte ptr[ToggleTurn], NO
            jmp short     redisplay
setToggleTurn:
            mov     byte ptr[ToggleTurn], YES
            jmp short     redisplay
;------------------------------
getpixel:
            mov     ah, 0dh
            mov     bh, 0                   ; page
            int     10h                     ; al = getpixel(cx, dx)
            ret
putpixel:
            mov     ah, 0ch
            mov     bh, 0                   ; page
            int     10h                     ; putpixel(cx, dx, al)
            ret
;------------------------------
pixel       db      0
begininx    dw      0
inx         dw      320
iny         dw      200
Saved_File_Pos dw   0
;------------------------------
dontToggleTurn:
;------------------------------
; Restore trashed palette
;------------------------------
            cmp     AL, ' '
            jne     notrestorepal
            mov     si, offset header_palet
            call    writepalette
            jmp     menu
notrestorepal:
;------------------------------
; Trash the palette (for test)
;------------------------------
            cmp     AL, 't'
            jne     nottrashpal
            mov     si, offset header_palet ; ref 1 (first)
            push    es
            mov     dx,40h
            push    dx
            pop     es
            mov     ax, word ptr[es: 06ch]      ; timer low word

            add     si, ax
            pop     es
            call    writepalette
            jmp     menu
nottrashpal:
;------------------------------
; Toggle count colors
;------------------------------
            cmp     AL, '$'
            jne     notdollar
            cmp     byte ptr[ToggleCount], NO
            je      ToggleCountis0
            mov     byte ptr[ToggleCount], NO
            jmp     redisplay
ToggleCountis0:
            mov     byte ptr[ToggleCount], YES
            jmp     redisplay
notdollar:
;------------------------------
; dump vesa info
;------------------------------
            cmp     AL, 'v'
            jne     notvesadump
            call    gotoxy
            call    setcolor
            call    DumpVesaInfo
            jmp     menu
notvesadump:
;------------------------------
; Are we in command line mode?
;------------------------------
            cmp     byte ptr[OneFileOnly], 1
            jne     notcommandline
            jmp     menu
;------------------------------
; End of command line processing.
;------------------------------
notcommandline:
            mov     byte ptr[cs: Zoom], 0

            cmp     ah, 048h                 ; Up?
            jne     notup
            mov     ax, [File_Pos]
            cmp     ax, 0
            jnbe    no_menu
            jmp     menu
no_menu:
            dec     ax
            mov     [File_Pos], ax
            mov     bl, 13
            mul     bl                       ; ax = al*byte
            mov     si, offset filestore
            add     si, ax
            jmp     store_para_list
;------------------------------
notup:
            cmp     ah, 047h                 ; Home?
            jne     nothome
            cmp     word ptr[File_Pos], 0
            jne     not_menu
            jmp     menu
not_menu:
            mov     word ptr[File_Pos], 0
            mov     si, offset filestore
            jmp     store_para_list
;------------------------------
nothome:
            cmp     ah, 4fh                  ; End?
            jne     notend
            mov     ax, [GIFFileCounter]
            dec     ax
            cmp     word ptr[File_Pos], ax
            jne     not_last_yet
            jmp     menu
not_last_yet:
            mov     word ptr[File_Pos], ax
            mov     bl, 13
            mul     bl                       ; ax = al*byte
            mov     si, offset filestore
            add     si, ax
            jmp     store_para_list
;------------------------------
notend:
            cmp     ah, 051h                 ; PgDn?
            jne     notpgdn
            mov     ax, [File_Pos]
            inc     ax
            cmp     ax, [GIFFileCounter]     ; last pic anyway?
            jb      nmenu
            jmp     menu
nmenu:
            dec     ax
            add     ax, 10
            cmp     ax, [GIFFileCounter]     ; beyond end?
            jna     stillinrange
            mov     ax, [GIFFileCounter]
            dec     ax
stillinrange:
            mov     [File_Pos], ax
            mov     bl, 13
            mul     bl                       ; ax = al*byte
            mov     si, offset filestore
            add     si, ax
            jmp     store_para_list
;------------------------------
notpgdn:
            cmp     ah, 049h                 ; PgUp?
            jne     notpgup
            mov     ax, [File_Pos]
            cmp     ax, 0
            jne     no_a_menu
            jmp     menu
no_a_menu:
            sub     ax, 10
            cmp     ax, 0
            jg      stillok                 ; signed
            mov     ax, 0
stillok:
            mov     [File_Pos], ax
            mov     bl, 13
            mul     bl                      ; ax = al*byte
            mov     si, offset filestore
            add     si, ax
            jmp     store_para_list
notpgup:
;------------------------------
; Next Pic
;------------------------------
            mov     ax, [cs: File_Pos]
            inc     ax
            cmp     ax, [cs: GIFFileCounter]
            jae     byebye
;            inc     ax
            mov     [cs: File_Pos], ax
            mov     bl, 13
            mul     bl                      ; ax = al*byte
            mov     si, offset filestore
            add     si, ax
            jmp     store_para_list


;------------------------------------------------------------
infoGIF:
            mov     ax, cs
            mov     es, ax
            mov     ds, ax
            mov     dx, offset Last_GIF_Shown
            mov     ah, 9
            int     21h
            call    displaylastGIFname
            mov     dx,offset newline
            mov     ah,9
            int     21h

            ret

;------------------------------
; Good ending
;------------------------------
returnvalue db      -1
;------------------------------

byebye:
            push    cs
            pop     ds

            mov     byte[returnvalue], 0  ; good ending
            call    textmode
            call    infoGIF

            jmp short     endprogram
;------------------------------
endwitherror:
            push    cs
            pop     ds

            push    dx                      ; save incoming message
            mov     byte[returnvalue], 1    ; bad ending

            call    textmode
            call    infoGIF

            pop     dx                      ; restore incoming message
            mov     ah, 9
            int     21h                     ; DOS - PRINT STRING
                                            ; DS:DX -> string terminated by "$"
endprogram:
            call    DebugMode

            mov     al, byte[returnvalue]
            mov     ah, 4ch
            int     21h                     ; DOS - 2+ - QUIT
                                            ; AL = exit code
;-------- DisplayLastGIFName -----------------------------------------------
displaylastGIFname:
            pusha
            mov     si, offset paramlist
            dec     si
boucle:
            inc     si
            cmp     byte ptr[si], 0
            jne     boucle
            mov     byte ptr[si], "$"
            mov     dx, offset paramlist
            mov     ah, 9
            int     21h
            popa
            ret
;------ gotoxy(); -------------
gotoxy:
            mov     ah, 2
            mov     bh, 0
            mov     dh, 2                   ; y: 1 => 0, 2 => 1, ...
            mov     dl, 0                   ; x: 1 => 0
            int     10h
            ret

;------------------------------
; Data
;------------------------------
aster       db      0,0,0,0,0,0,0,0
inder       dw      0
;------------------------------
; Converts the unsigned integer value in AX to a string of digits
; and sends it to screen.
utoa:
            pusha
            mov     word ptr[inder], 0
            call    puti2
            mov     ah, 9
            mov     dx, offset aster
            int     21h
            popa
            ret
; Recursive routine to store the value in AX as an integer.
puti2:
            mov     bx, 10
            xor     dx, dx
            div     bx                      ; the one and only div
            or      ax, ax                  ;is ax==0?
            jz      pdone
            push    dx
            call    puti2
            pop     dx
pdone:      mov     al, dl
            or      al, '0'
            call    putit
            ret
; PutIt: - Writes the AL in AL to the "aster" buffer,
; - '$' terminates the string,
; - Increments Inder.
putit:
            mov     di, [inder]
            mov     [aster+di], al
            mov     byte ptr[aster+1+di], '$'
            inc     word ptr[inder]
            ret

;---------------------------------------------------------
; AX word to Ascii hex
;---------------------------------------------------------
m_BaseType      db      16      ; 10 = decimal, 16 = hexa
m_text          db      "xxxx$"
;------------------------------
utoh:
                push    ax
                mov     al,ah
                mov     di, offset m_text+1
                call    Calc

                pop     ax
                mov     di, offset m_text+3
                call    Calc

                mov     dx,offset m_text
                mov     ah,9
                int     21h

                ret
calc:
                mov     bl, byte ptr [cs: m_BaseType]
                mov     cx,2
LoopCalc:
                xor     ah, ah
                div     bl              ; ax/10   or   ax/16
                                        ; al quo, ah rem
                cmp     ah, 10
                jb      NotHexa
                add     ah, '7'         ; make it an hexa number
                jmp short     NotDec
NotHexa:
                add     ah, '0'         ; make it an ascii number
NotDec:
                ;mov     byte ptr cs:[ di], ah
                cs mov     byte ptr [ di], ah
                dec     di
                loop     LoopCalc

                ret

;------------
; Data.
;------------
Last_GIF_Shown          db      0dh,0ah
                        db      'Last GIF : $'
Out_of_memory           db      'Out of memory.$'
File_not_found          db      'File not found.$'
File_read_error         db      'File read error.$'
Img_block_not_found     db      'Image block not found in the GIF file.$'
Error_in_GIF_file       db      'Error in the GIF file.$'
Not_a_GIF_file          db      'Not a GIF file.$'
Vga_not_found           db      'VGA not found.$'
VESA_not_found          db      'VESA not found.$'
;------------
CardValueIs            db      0dh,0ah
                       db      0dh,0ah,'Window size/granularity is $'
CardshiftValueIs       db      ' (1 << $'

lastVideoMode           db      0dh,0ah
                        db      'Video mode: $'
info1                   db      0dh,0ah
                        db      0dh,0ah
                        db      'VESA info:',0dh,0ah
                        db      0dh,0ah
                        db      'Window granularity in kb : $'
infoname                db      0dh,0ah
                        db      'Signature : "'
sign                    db      '????'
                        db      '"   Version : '
                        db      '$'
dot                     db      '.$'
newline                 db      0dh,0ah,'$'
picture_number          db      ' ($'
out_of                  db      '/$'
out_of_end              db      ') $'
info2                   db      0dh,0ah
                        db      'Bytes per scan line : $'
info3                   db      0Dh,0ah
                        db      'Window size in kb : $'

infowa                  db      0Dh,0ah
                        db      'Window attributes window A : $'
infowb                  db      0Dh,0ah
                        db      'Window attributes window B : $'
infowaseg               db      0dh,0ah
                        db      'Start segment of window A : $'
infowbseg               db      0dh,0ah
                        db      'Start segment of window B : $'
videomemory             db      0dh,0ah
                        db      0Dh,0ah
                        db      'Video memory in k : $'
goodBye                 db      0Dh,0ah,0dh,0ah
                        db      0Dh,0ah
                        db      'Done.$'
directcall              db      0dh,0ah
                        db      'window direct bank switch : $'

; mode_attributes bits -- VBE v2.0 only
notsup                  db     0dh,0ah, 'Bank switched mode not supported!$'
linear                  db     0dh,0ah, 'Linear framebuffer mode supported$'

VBEv20                  db      0

;------------

DebugMode:
        push    es
        mov     dx,40h
        push    dx
        pop     es
        ;mov     ax, [es: 017h]
        es mov     ax, word[017h]
        pop     es

        test    al, 2
        jnz     DumpVesaInfo ; 2 - Left Shift key depressed
        test    al, 1
        jnz     DumpVesaInfo ; 1 - Right Shift key depressed
        ret

DumpVesaInfo:
        mov     dx,offset lastVideoMode
        mov     ah, 9
        int     21h
        ;mov     ax,word ptr[cs: VideoMode]
        cs mov     ax,[VideoMode]
        call    utoa
        mov     ah, 2
        mov     dl, ' '
        int     21h
        mov     ah, 2
        mov     dl, '('
        int     21h

        ;mov     ax,word ptr[cs: VideoMode]
        cs mov     ax,[VideoMode]
        call    utoh
        mov     ah, 2
        mov     dl, 'h'
        int     21h
        mov     ah, 2
        mov     dl, ')'
        int     21h

        call    Get_SVGA_info
        call    Get_SVGA_Mode_info

        mov     dx,offset CardValueIs
        mov     ah, 9
        int     21h
        ;mov     ax,word ptr[cs: Card]
        cs mov     ax,[Card]
        call    utoa

        mov     dx,offset CardshiftValueIs
        mov     ah, 9
        int     21h

        ;mov     ax,word ptr[cs: Cardshift]
        cs mov     ax,[Cardshift]
        call    utoa

        mov     ah, 2
        mov     dl, ')'
        int     21h

        ret
;------------
Get_SVGA_info:
        pusha

; [is this "VBE2" needed? we are not using linear frame buffer... yet!]
        mov     byte ptr[signature_VESA+0],"V"   
        mov     byte ptr[signature_VESA+1],"B"
        mov     byte ptr[signature_VESA+2],"E"
        mov     byte ptr[signature_VESA+3],"2"

        mov     ax, 4F00h
        push    ds
        pop     es
        mov     di,offset SVGA_information
        int     10h
        cmp     ax, 4Fh
        jne     Get_SVGA_info_failed
;Get_SVGA_info_success:
        mov     dx,offset videomemory
        mov     ah, 9
        int     21h

        mov     ax,word ptr[cs: total_amount_of_video_memory_in_64k_blocks]
        xor     dx,dx
        mov     bx,64
        mul     bx

        call    utoa
        mov     si,offset signature_VESA
        mov     di,offset sign
        mov     cx,4
        rep     movsb
        mov     dx,offset infoname
        mov     ah, 9
        int     21h

; Check VBE 2.0:
        xor     ax,ax
        mov     al,byte ptr[cs: VESA_version_number+1]
        cmp     al,2
        jne     notvbe2
        mov     byte ptr[cs: VBEv20], 1
notvbe2:
        call    utoa
        mov     dx,offset dot
        mov     ah, 9
        int     21h
        xor     ax,ax
        mov     al,byte ptr[cs: VESA_version_number]
        call    utoa

Get_SVGA_info_failed:
        popa
        ret

;------------
GetSVGAModeinfo:
        pusha
        mov     word ptr[cs: Card], 1
        mov     word ptr[cs: CardShift], 0

        mov     ax, 4F01h
        ;mov     cx, word ptr[cs: VideoMode]
        cs mov     cx, [VideoMode]
        push    ds
        pop     es
        mov     di, offset MODE_information ; 256-byte buffer
        int     10h
        cmp     ax, 4Fh         ; if function supported
        jne     GetSVGAModeinfofailed

        mov     ax,word ptr[cs: window_size_in_kb]         ; 64k?
        cwd
        ;mov     bx,word ptr[cs: window_granularity_in_KB] ; 64k? 4k?
        cs mov     bx, [window_granularity_in_KB]          ; 64k? 4k?
        cmp     bx,0
        jz      GetSVGAModeinfofailed
        div     bx

        mov     word ptr[cs: Card], ax

        mov     bx, 64
        mov     cx, 6
morebnkshift:
        cmp     ax, bx
        je      foundbnkshift
        shr     bx, 1
        loop    morebnkshift

foundbnkshift:
        mov     word ptr[cs: Cardshift],cx

        popa
        ret

GetSVGAModeinfofailed:
        popa

        mov     dx, offset VESA_not_found ; 'VESA not found$'
        jmp     endwitherror

;--------------------
Get_SVGA_Mode_info:
        pusha
        mov     ax, 4F01h
        ;mov     cx, word ptr[cs: VideoMode]
        cs mov     cx, [VideoMode]
        push    ds
        pop     es
        mov     di, offset MODE_information ; 256-byte buffer
        int     10h
        cmp     ax, 4Fh
        je      getsuccess
        jmp     getfailed

getsuccess:
        mov     dx,offset info1
        mov     ah, 9
        int     21h
        mov     ax,word ptr[cs: window_granularity_in_KB]
        call    utoa

        mov     dx,offset info3
        mov     ah, 9
        int     21h
        mov     ax,word ptr[cs: window_size_in_kb]
        call    utoa

        mov     dx,offset info2
        mov     ah, 9
        int     21h
        mov     ax,word ptr[cs: bytes_per_scan_line]
        call    utoa

        mov     dx,offset infowa
        mov     ah, 9
        int     21h
        xor     ax,ax
        mov     al,byte ptr[cs:window_attributes_window_A]
        call    utoa

        mov     dx,offset infowb
        mov     ah, 9
        int     21h
        xor     ax,ax
        mov     al,byte ptr[cs:window_attributes_window_B]
        call    utoa

        mov     dx,offset infowaseg
        mov     ah, 9
        int     21h
        mov     ax,word ptr[cs: start_segment_of_window_A]
        call    utoh
        mov     ah, 2
        mov     dl, 'h'
        int     21h

        mov     dx,offset infowbseg
        mov     ah, 9
        int     21h
        mov     ax,word ptr[cs: start_segment_of_window_B]
        call    utoh
        mov     ah, 2
        mov     dl, 'h'
        int     21h

        mov     dx,offset directcall
        mov     ah, 9
        int     21h
        mov     ax,word ptr[cs: window_bank_switch+2]
        call    utoh

        mov     ah, 2
        mov     dl, ':'
        int     21h

        mov     ax,word ptr[cs: window_bank_switch]
        call    utoh

; Check VBE 2.0:
        cmp     byte ptr[cs: VBEv20], 1
        jne     getfailed
        mov     ax, word ptr[cs: mode_attributes]
        push    ax
        test    ax, 64
        jz      supok      ; bit not set

        mov     dx,offset notsup
        mov     ah, 9
        int     21h
supok:
        pop     ax
        test    ax,128
        jz      getfailed      ; bit not set

        mov     dx,offset linear
        mov     ah, 9
        int     21h
getfailed:
        popa
        ret

;------------
SVGA_information:
signature_VESA                                db   "VBE2"
VESA_version_number                           dw   0
pointer_to_OEM_name                           dd   0
capabilities_flags                            dd   0
pointer_to_list_of_supported_VESA_video_modes dd   0
total_amount_of_video_memory_in_64k_blocks    dw   0
                                              db 236 dup (0)
;                                             times 236 db 0
OEM_software_version                          dw   0
pointer_to_vendor_name                        dd   0
pointer_to_product_name                       dd   0
pointer_to_product_revision_string            dd   0
                                              db 478 dup (0)
;                                             times 478 db 0

;------------
MODE_information:
mode_attributes                              dw         0
window_attributes_window_A                   db         0
window_attributes_window_B                   db         0
window_granularity_in_kb                     dw         0
window_size_in_kb                            dw         0
start_segment_of_window_A                    dw         0
start_segment_of_window_B                    dw         0
window_bank_switch                       dd         0
bytes_per_scan_line                          dw         0
width_in_pixels_graphics_or_characters_text  dw         0
height_in_pixels_graphics_or_characters_text dw         0
width_of_character_cell_in_pixels            db         0
height_of_character_cell_in_pixels           db         0
number_of_memory_planes                      db         0
number_of_bits_per_pixel                     db         0
number_of_banks                              db         0
memory_model_type                            db         0
size_of_bank_in_kb                           db         0
number_of_image_pages                        db         0
                                             db         0
red_mask_size                                db         0
red_field_position                           db         0
green_mask_size                              db         0
green_field_size                             db         0
blue_mask_size                               db         0
blue_field_size                              db         0
reserved_mask_size                           db         0
reserved_mask_position                       db         0
direct_color_mode_info                       db         0

physical_address_of_linear_video_buffer      dd         0
pointer_to_start_of_offscreen_memory         dd         0
kb_of_offscreen_memory                       dw         0
                                             db 206 dup (0)
;                                            times 206 db 0

FinalByte:
;---------
; TASM stuff:
;kgif            ends
;                end     start
;---------                                         ---------
